<?php

/* search.php: Interface for searching mailboxes
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: search.php,v 1.101 2005/09/28 15:15:22 sailfrog Exp $
*/

require_once('../lib/wrapper_functions.php');
$start = hm_time();

/* collect and validate incoming vars */
$hm_input = hm_validate_input($_POST, $_GET, 'search');

/* set ini values and check/get the SESSION vars */
if (hm_session_continue($hm_input['id'], $config_file)) {
    $config = $_SESSION['config'];
}
if (isset($_SESSION['settings']['template_set'])) {
    $template_set = $_SESSION['settings']['template_set'];
}
elseif (isset($config['template_set']) && $config['template_set'] != '') {
    $template_set = $config['template_set'];
}
else {
    $template_set = 'default';
}

$hm_input['id'] = session_id();
$str = $_SESSION['str'];
$not_match = false;

if (isset($hm_input['destination'])) {
    $dest = $hm_input['destination'];
}
elseif (isset($_SESSION['move_to_destination'])) {
    $dest = $_SESSION['move_to_destination'];
}
else {
    $dest = 'INBOX';
}
$_SESSION['move_to_destination'] = $dest;

if ((isset($_SESSION['settings']['simple']) && $_SESSION['settings']['simple'] == 'true')
   || (isset($_SESSION['auto_simplify']) && $_SESSION['auto_simplify'] == 'true')) {
    $simple = $_SESSION['settings']['simple'];
}
else {
    $simple = false;
}

/* login to imap */
if (!$imap_stream = hm_imap_login($_SESSION['user'], $_SESSION['pass'], $config,
                                  $hasty_key, false, false)) {
    hm_logged_out($config_file);
}

/* do message control actions */
$action = false;
switch (true) {
    case isset($hm_input['move']):
        $action = 'move';
        break;
    case isset($hm_input['sort']):
        $action = 'sort';
        break;
    case isset($hm_input['delete']):
        $action = 'delete';
        break;
    case isset($hm_input['read']):
        $action = 'read';
        break;
    case isset($hm_input['flag']):
        $action = 'flag';
        break;
    case isset($hm_input['unflag']):
        $action = 'unflag';
        break;
    case isset($hm_input['unread']):
        $action = 'unread';
        break;
    case isset($hm_input['expunge']):
        $hm_input['selection_type'] = 'all';
        $action = 'expunge';
        break;
    case isset($hm_input['copy']):
        $action = 'copy';
        break;
    default:
        break;
}
if ($action) {
    if ($hm_input['selection_type'] == 'page') {
        $sel = false;
        $array = $_SESSION['search_info'];
    }
    elseif ($hm_input['selection_type'] == 'marked') {
        $array = $_POST;
        $sel = true;
    }
    elseif (isset($hm_input['selection_type'])) {
        if (isset($_SESSION['search_info']) && 
            array_key_exists($hm_input['selection_type'], $_SESSION['search_info'])) {
            $array = array($hm_input['selection_type'] => 
                           $_SESSION['search_info'][$hm_input['selection_type']]);
            $sel = false;
        }
    }
    foreach ($array as $name => $value) {
            if ($sel) {
                $name = base64_decode($name);
            }
        if (substr($name, 0, 5) == 'fold_' || !$sel) {
            if ($sel) {
                $fold = substr($name, 5);
            }
            else {
                $fold = $name;
            }
            $dsp_count = count($value);
            if ($dsp_count > 1) {
                $dsp_end = $str[18];
            }
            else {
                $dsp_end = $str[26];
            }
            $dest = false;
            if (isset($hm_input['destination'])) {
                $dest = $hm_input['destination'];
            }
            if ($sel) {    
                $uid_string = $value[0];
                if ($dsp_count > 1) {
                    $uid_string = implode(',', $value);
                }
            }
            else {
                $uids = array_keys($value);
                $uid_string = $uids[0]; 
                if ($dsp_count > 1) {
                    $uid_string = implode(',', $uids);
                }
            }
            hm_select_mailbox($imap_stream, $fold);
            if ($action == 'delete' && isset($hm_input['skip_trash'])) {
                $skip = true;
            }
            else {
                $skip = false;
            }
            $result = hm_message_controls ($action, $imap_stream,
                                           $uid_string, $dest, false, false, $fold, 
                                           $dsp_count, $dsp_end, $skip);
        }
    }
    $hm_input['update'] = 1;        
}

/* check for keep as new setting */
if (isset($hm_input['keep_as_new'])) {
    hm_select_mailbox($imap_stream, $hm_input['kan_folder'], false, true);
    hm_imap_action($imap_stream, $hm_input['kan_uid'], 'UNREAD', false, false);
}

/* get unseen messages */
$unseen = hm_unseen_status ($imap_stream, $_SESSION['folders']);

/* check for search -> filter promotion */
/*if (isset($hm_input['promote'])) {
    $_SESSION['search_to_filter'][] =  $_SESSION['search_params']['search_string'].' '.$hm_input['sstring'];
}*/

/* check the session and reset buttons */
if (isset($hm_input['reset'])) {
    if (isset($_SESSION['search_params'])) {
        $_SESSION['search_params'] = array();
        unset($_SESSION['search_params']);
        unset($_SESSION['search_info']);
    }
    foreach($hm_input as $key => $value) {
        if (substr($key, 0, 4) == 'FLAG') {
            unset($hm_input[$key]);
        }
    }
}

$new_search = false;
$old_search = false;

/* execute the submitted search */
if (isset($hm_input['update']) || isset($hm_input['search'])) {

    $range_type  = $hm_input['range_type'];
    $day         = $hm_input['day'];
    $month       = $hm_input['month'];
    $year        = $hm_input['year'];
    $size        = $hm_input['size'];
    $size_unit   = $hm_input['size_unit'];
    $search_part = $hm_input['search_part'];
    $sstring     = $hm_input['sstring'];
    $size_type   = $hm_input['size_type'];
    $search_folder = array();
    $_SESSION['search_params'] = $hm_input;
    unset($_SESSION['search_params']['mailbox']);
    unset($_SESSION['search_params']['id']);
    $date_string = '';
    $size_string = '';
    $flag_string = '';

        /* check which folders to search */
        $search_folder = array();
        if (!isset($hm_input['search_folders'])) {
            $search_folder[] = 'INBOX';
        }
        elseif (is_array($hm_input['search_folders']) && 
                !empty($hm_input['search_folders'])) {
            $search_folder = $hm_input['search_folders'];
        }
        else {
            $search_folder[] = 'INBOX';
        }
        $_SESSION['search_params']['search_folder'] = $search_folder;
    
        /* size options */
        if (isset($hm_input['size_type']) && $hm_input['size_type'] != 'any'
            && isset($hm_input['size']) && $hm_input['size'] > 0) {
            $size_string = $hm_input['size_type'].' ';
            switch ($hm_input['size_unit']) {
                case 'b':
                    $size_string .= $hm_input['size'];
                    break;
                case 'k';
                    $size_string .= $hm_input['size']*1024;
                    break;
                case 'm';
                    $size_string .= $hm_input['size']*1048576;
                    break;
            }
        }
    
        /* date range options */
        if (isset($hm_input['range_type']) && $hm_input['range_type'] != 'any') {
            $date_string = $hm_input['range_type'].' '.$hm_input['day'].'-'.
                           $hm_input['month'].'-'.$hm_input['year'];    
        }
    
        /* look for flag options */
        foreach ($hm_input as $key => $val) {
            if (substr($key, 0, 4) == 'FLAG') {
                $flag_string .= substr($key, 5).' ';
            }
        }
        $flag_string = rtrim($flag_string);
        if (isset($hm_input['sstring']) && $hm_input['sstring'] != '') {
            $spart = $hm_input['search_part'];
            $string = $hm_input['sstring'];
        }
        else {
            $spart = '';
            $string = false;
        }
        if ($string && isset($hm_input['not_match'])) {
            $_SESSION['search_params']['not_match'] = 1;
            $spart = 'NOT '.$spart;
            $not_match = true;
        }
        $search_string = 'SEARCH '.$flag_string.' '.$date_string.' '.$size_string.' '
                         .$spart;
        $search_string = preg_replace("/\s{2,}/", ' ', $search_string);
        if (rtrim($search_string) == 'SEARCH') {
            $search_string .= 'ALL';
        }
        $data = array();
        $f_cnt = 0;
        $ht_cnt = 0;
        
        /* perform search and get headers for resulting UIDs */
        foreach($search_folder as $sf) {
            $f_cnt++;
            hm_select_mailbox($imap_stream, $sf);
            $uids = hm_search_mailbox($imap_stream, $search_string, $string);
            $uids = str_replace(' ', ',', trim($uids));
            $headers = &hm_get_messages($uids, $imap_stream);
            if (!empty($headers)) {
                $ht_cnt += count($headers);
                uasort($headers, "hm_date_cmp");
                $data[$sf] = $headers;
            }
        }
    
        /* save search in session */
        $_SESSION['search_info'] = $data;
        $_SESSION['search_params']['search_string'] = $search_string;
        $_SESSION['search_params']['ht_cnt'] = $ht_cnt;
        $_SESSION['search_params']['f_cnt'] = $f_cnt;
        $new_search = true;
}

/* reload previous search */
elseif (isset($_SESSION['search_params']) && !empty($_SESSION['search_params']) &&
        isset($_SESSION['search_info'])) {
    foreach ($_SESSION['search_params'] as $key => $value) {
        if (substr($key, 0, 4) == 'FLAG') {
            $hm_input[$key] = $value;
        }
    }
    $range_type    = $_SESSION['search_params']['range_type'];
    $day           = $_SESSION['search_params']['day'];
    $month         = $_SESSION['search_params']['month'];
    $year          = $_SESSION['search_params']['year'];
    $size          = $_SESSION['search_params']['size'];
    $size_unit     = $_SESSION['search_params']['size_unit'];
    $search_part   = $_SESSION['search_params']['search_part'];
    $sstring       = $_SESSION['search_params']['sstring'];
    $size_type     = $_SESSION['search_params']['size_type'];
    $search_folder = $_SESSION['search_params']['search_folder'];
    $data          = $_SESSION['search_info'];
    $search_string = $_SESSION['search_params']['search_string'];
    $ht_cnt        = $_SESSION['search_params']['ht_cnt'];
    $f_cnt         = $_SESSION['search_params']['f_cnt'];
    $old_search    = true;
    $not_match     = true;
}
else {
    $range_type  = 'any';
    $day         = '1'; 
    $month       = 'Jan';
    $year        = '1970';
    $size        = '';
    $size_unit   = 'b';
    $search_part = 'TEXT';
    $sstring     = '';
    $size_type   = 'any';
    $search_folder = array();
}

/* check for missing folders */
hm_check_folders($imap_stream);

/* toolbar */
$page_data = array();
list($page_data['td'], $page_data['hd']) = hm_toolbar2($imap_stream, $hm_input['mailbox'],
                                       $unseen, false, $str[11]);
                                   
if (isset($_SESSION['notices']) && !empty($_SESSION['notices'])) {
    $page_data['notices'] = $_SESSION['notices'];
    $_SESSION['notices'] = array();
}
else {
    $page_data['notices'] = false;
}

$page_data['nn4'] = isset($_SESSION['nn4_warning']);
$page_data['input_id'] = $hm_input['id'];
$page_data['input_mailbox'] = $hm_input['mailbox'];
$page_data['current_search'] = hm_htmlize($sstring);
$page_data['search_data'] = array();

/* search results */
$page_data['show_results'] = false;
if ($new_search || $old_search) {
    $page_data['show_results'] = true;
    $page_data['lbl_reset'] = $str[60];
    $page_data['lbl_update'] = $str[61];
    $page_data['lbl_selected'] = $str[6];
    $page_data['old_search'] = $old_search;
    $page_data['lbl_saved_result'] = $str[90];
    
    /* summary */
    $page_data['lbl_results_for'] = $str[63];
    $page_data['search_type'] = $search_string;
    $page_data['lbl_messages_found'] = $str[64];
    $page_data['lbl_folders_searched'] = $str[65];
    $page_data['result_count'] = $ht_cnt;
    $page_data['folder_count'] = $f_cnt;
    
    /* message controls */
    $t_all = false;
    if (isset($hm_input['toggle_all'])) {
        $t_all = true;
    }
    if (count($data) > 0) {
        $dropdown = array();
        $dropdown['marked'] = $str[6];
        $fold_tmp = array_keys($data);
        $fold_tmp_dsp = hm_folder_dsp($fold_tmp);
        foreach ($fold_tmp as $val) {
            $dropdown[$val] = $str[51].' '.$fold_tmp_dsp[$val];
        }
        $dropdown['page'] = $str[52];
        $page_data['mc'] = hm_print_message_controls2($dropdown, $dest, $simple);
        
        /* loop through folders with search results */
        $out_data = array();
        foreach($data as $folder => $uid_array) {
            $out_folder = array(
                'num_results' => count($uid_array),
                'url_folder' => urlencode($folder),
                'folder_name' => hm_htmlize($_SESSION['dsp_folders'][$folder]),
                
            );
            
            $chk_folder = base64_encode('fold_'.$folder);
            $out_folder['chk_folder'] = $chk_folder;
    
            /* get search result message data */
            $out_folder['rows'] = array();
            foreach ($uid_array as $uid => $headers) {
                $current = false;
                if (isset($hm_input['uid']) && $hm_input['uid'] == $uid) {
                    $current = true;
                }
                $out_folder['rows'][] = hm_print_row2($headers, $t_all, $folder, $hm_input['id'], $uid,
                            array(), true, $chk_folder, true, false, $current);
            }
            
            $out_data[] = $out_folder;
            
        }
        $page_data['search_data'] = $out_data;
    }
}
/* search form */
$months = $str['months'];
$page_data['lbl_target_folders'] = $str[55];
$page_data['lbl_msg_date_range'] = $str[56];
$page_data['lbl_msg_flags'] = $str[57];
$page_data['lbl_date_type'] = $str[289];
$page_data['lbl_date'] = $str[23];

$page_data['folder_options'] = hm_folder_options2($search_folder, false, $simple, false);

$range_types = array(
    'any' => $str[76],
    'SENTBEFORE' => $str[77],
    'SENTSINCE' => $str[78],
    'SENTON' => $str[79],
);

# range types
$page_data['range_types'] = $range_types;
$page_data['range_type'] = $range_type;

# months
$page_data['months'] = $months;
$page_data['month'] = $month;

# days
$days = array();
for ($i=1;$i<32;$i++) {
    $days[] = $i;
} 
$page_data['days'] = $days;
$page_data['day'] = $day;

# years
$years = array();
for ($i=1970;$i<2010;$i++) {
    $years[] = $i;
} 
$page_data['years'] = $years;
$page_data['year'] = $year;

$cbs = array (
    'FLAG_ANSWERED' => array ( $str[67], isset($hm_input['FLAG_ANSWERED']) ),
    'FLAG_UNANSWERED' => array ( $str[68], isset($hm_input['FLAG_UNANSWERED']) ),
    'FLAG_SEEN' => array ( $str[314], isset($hm_input['FLAG_SEEN']) ),
    'FLAG_UNSEEN' => array ( $str[315], isset($hm_input['FLAG_UNSEEN']) ),
    'FLAG_DELETED' => array ( $str[89], isset($hm_input['FLAG_DELETED']) ),
    'FLAG_UNDELETED' => array ( $str[70], isset($hm_input['FLAG_UNDELETED']) ),
    'FLAG_FLAGGED' => array ( $str[71], isset($hm_input['FLAG_FLAGGED']) ),
    'FLAG_UNFLAGGED' => array ( $str[72], isset($hm_input['FLAG_UNFLAGGED']) ),
    'FLAG_DRAFT' => array ( $str[69], isset($hm_input['FLAG_DRAFT']) ),
    'FLAG_UNDRAFT' => array ( $str[73], isset($hm_input['FLAG_UNDRAFT']) ),
    'FLAG_RECENT' => array ( $str[74], isset($hm_input['FLAG_RECENT']) ),
    'FLAG_OLD' => array ( $str[75], isset($hm_input['FLAG_OLD']) ),
);
$page_data['message_flags'] = $cbs;

$page_data['lbl_size'] = $str[24];
$page_data['lbl_size_type'] = $str[290];

$size_types = array(
    'any' => $str[76],
    'SMALLER' => $str[80],
    'LARGER' => $str[81],
);
$page_data['size_types'] = $size_types;
$page_data['size_type'] = $size_type;
$page_data['html_size'] = hm_htmlize($size);

$size_units = array(
    'b' => $str[82],
    'k' => $str[83],
    'm' => $str[84],
);
$page_data['size_units'] = $size_units;
$page_data['size_unit'] = $size_unit;

$page_data['lbl_search_string'] = $str[58];
$page_data['simple_search'] = (isset($config['simple_search']) &&
                              $config['simple_search'] == 'true');
$page_data['not_match'] = isset($_SESSION['search_params']['not_match']);
$page_data['lbl_not_match'] = $str[346];
$page_data['lbl_section'] = $str[59];

$search_parts = array(
    'TEXT' => $str[85],
    'FROM' => $str[22],
    'TO' => $str[25],
    'CC' => $str[87],
    'BCC' => $str[88],
    'SUBJECT' => $str[21],
    'BODY' => $str[86],
);

$page_data['search_parts'] = $search_parts;
$page_data['search_part'] = $search_part;
$page_data['subject_lbl'] = $str[21];
$page_data['from_lbl'] = $str[22];
$page_data['date_lbl'] = $str[23];
$page_data['size_lbl'] = $str[24];
$page_data['lbl_search'] = $str[11];
$page_data['lbl_reset'] = $str[60];
$page_data['start_time'] = $start;
if (isset($_SESSION['settings']['folder_size'])) {
    $page_data['folder_size'] = $_SESSION['settings']['folder_size'];
}
else {
    $page_data['folder_size'] = 140;
}

hm_imap_logout($imap_stream); 

/* output HTTP headers */
hm_output_http_headers();

/* output page */
hm_output_page($page_data, 'search_template.php', $template_set);

?>
